/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistableElement;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.DataChangeBridge;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * <p>TaxonEditorInput class.</p>
 *
 * @author n.hoffmann
 * @created 19.03.2009
 * @version 1.0
 */
public class TaxonEditorInput implements IEditorInput, IConversationEnabled, IPersistableElement {

	private ConversationHolder conversation;

	private TaxonNode taxonNode;

	private TaxonEditorInputDataChangeBehaviour dataChangeBehavior;

	private TaxonBase initiallySelectedTaxonBase;

	private TaxonEditorInput(TaxonNode taxonNode, ConversationHolder conversation){
		this.conversation = conversation;
		this.taxonNode = taxonNode;
	}



	/**
	 * <p>NewInstance</p>
	 *
	 * @param taxonNodeUuid a {@link java.util.UUID} object.
	 * @return a {@link eu.etaxonomy.taxeditor.editor.TaxonEditorInput} object.
	 * @throws java.lang.Exception if any.
	 */
	public static TaxonEditorInput NewInstance(UUID taxonNodeUuid) throws Exception{
		try{
			ConversationHolder conversation = CdmStore.createConversation();
			return NewInstance(taxonNodeUuid, conversation);
		}catch(Exception e){
			throw e;
		}
	}

    /**
     *
     * @param taxonNodeUuid
     * @param conversation
     * @return
     */
    private static TaxonEditorInput NewInstance(UUID taxonNodeUuid, ConversationHolder conversation){


    	TaxonNode taxonNode = CdmStore.getService(ITaxonNodeService.class).load(taxonNodeUuid, null);

		if(taxonNode == null){
			MessagingUtils.warningDialog("Not yet implemented", TaxonEditorInput.class, "Selected element is not type TaxonBase.");
			return null;
		}

    	return new TaxonEditorInput(taxonNode, conversation);
    }

    /**
     * <p>NewInstanceFromTaxonBase</p>
     *
     * @param taxonBaseUuid a {@link java.util.UUID} object.
     * @return a {@link eu.etaxonomy.taxeditor.editor.TaxonEditorInput} object.
     */
    public static TaxonEditorInput NewInstanceFromTaxonBase(UUID taxonBaseUuid){
    	ConversationHolder conversation = CdmStore.createConversation();

    	TaxonEditorInput input = null;

    	TaxonBase taxonBase = CdmStore.getService(ITaxonService.class).find(taxonBaseUuid);
    	if (taxonBase != null){
	    	if(taxonBase.isOrphaned()) {
	    		MessagingUtils.warningDialog("Orphaned Taxon", TaxonEditorInput.class, "This is an orphaned taxon i.e. a taxon that is not connected to a classification and not having any taxonomic relationships. Editing of orphaned taxon is currently not supported.");
	    	}
	    	else if(taxonBase.isInstanceOf(Taxon.class)){
	    		Taxon taxon = CdmBase.deproxy(taxonBase, Taxon.class);
	
	    		if (taxon.getTaxonNodes().size() == 0 && taxon.isMisapplication()){
	    			// TODO get accepted taxon
	    			MessagingUtils.info("trying to open Mispplied Name ");
	
	    			Set<Taxon> acceptedTaxa = new HashSet<Taxon>();
	    			Set<TaxonRelationship> relations = taxon.getRelationsFromThisTaxon();
	    			for(TaxonRelationship relation : relations){
	    				if(relation.getType().equals(TaxonRelationshipType.MISAPPLIED_NAME_FOR())){
	    					acceptedTaxa.add(relation.getToTaxon());
	    				}
	    			}
	    			input =  getInputForMultipleTaxa(conversation, acceptedTaxa);
	
	    		}else{
	    			input = getInputForMultipleNodes(conversation, taxon.getTaxonNodes());
	    		}
	    	}else if(taxonBase instanceof Synonym){
	    		Synonym synonym = (Synonym) taxonBase;
	
	    		Set<Taxon> taxa = synonym.getAcceptedTaxa();
	    		input = getInputForMultipleTaxa(conversation, taxa);
	    	}
	    	if (input != null){
	    		input.setInitiallySelectedTaxonBase(taxonBase);
	    	}
    	}
    	

    	return input;
    }

	private static TaxonEditorInput getInputForMultipleNodes(ConversationHolder conversation, Set<TaxonNode> taxonNodes){
    	if(taxonNodes.size() == 1){
    		TaxonNode taxonNode = taxonNodes.iterator().next();
    		return NewInstance(taxonNode.getUuid(), conversation);
    	}else if(taxonNodes.size() > 1){
			TaxonNode taxonNode = ChooseFromMultipleTaxonNodesDialog.choose(taxonNodes);
			if(taxonNode != null){
				return NewInstance(taxonNode.getUuid(), conversation);
			}
		}else if(taxonNodes.size() == 0){
			// this is an undesired state
			MessagingUtils.warningDialog("Incorrect state", TaxonEditorInput.class, "The accepted taxon is not part of any classification. This should not have happened.");
		}
    	return null;
    }

    private static TaxonEditorInput getInputForMultipleTaxa(ConversationHolder conversation, Set<Taxon> taxa){
    	if(taxa.size() == 1){
    		Taxon taxon = taxa.iterator().next();
    		Set<TaxonNode> nodes = taxon.getTaxonNodes();
    		return getInputForMultipleNodes(conversation, nodes);
    	}else if(taxa.size() > 1){
    		Set<TaxonNode> taxonNodes = new HashSet<TaxonNode>();
			for ( Taxon taxon : taxa ){
				taxonNodes.addAll(taxon.getTaxonNodes());
			}
			return getInputForMultipleNodes(conversation, taxonNodes);
		}else if(taxa.size() == 0){
			// this is an undesired state
			MessagingUtils.warningDialog("Incorrect state", TaxonEditorInput.class, "Trying to open accepted taxon for a synonym or misapplication but" +
					" no accepted taxa are present. This should not have happened.");
		}
    	return null;
    }

    /**
     * <p>NewEmptyInstance</p>
     *
     * @param parentNodeUuid a {@link java.util.UUID} object.
     * @return a {@link eu.etaxonomy.taxeditor.editor.TaxonEditorInput} object.
     */
    public static TaxonEditorInput NewEmptyInstance(UUID parentNodeUuid){
    	ConversationHolder conversation = CdmStore.createConversation();

		TaxonNameBase<?, ?> name = PreferencesUtil.getPreferredNomenclaturalCode().getNewTaxonNameInstance(null);
		ITaxonTreeNode parentNode = CdmStore.getService(IClassificationService.class).getTreeNodeByUuid(parentNodeUuid);

		Taxon newTaxon = Taxon.NewInstance(name, parentNode.getReference());
		TaxonNode newTaxonNode = parentNode.addChildTaxon(newTaxon, parentNode.getReference(), parentNode.getMicroReference());

		// add the new taxon to the editors persistence context
		UUID newTaxonNodeUuid = CdmStore.getService(ITaxonNodeService.class).save(newTaxonNode);

		return new TaxonEditorInput(newTaxonNode, conversation);
    }

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#exists()
	 */
	/**
	 * <p>exists</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean exists() {
		return taxonNode != null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
	 */
	/**
	 * <p>getImageDescriptor</p>
	 *
	 * @return a {@link org.eclipse.jface.resource.ImageDescriptor} object.
	 */
	@Override
    public ImageDescriptor getImageDescriptor() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	/**
	 * <p>getName</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getName() {
		if(getTaxon() == null){
			return null;
		}
		TaxonNameBase<?, ?> name = getTaxon().getName();
		if (name == null || name.getTitleCache() == null) {
			return "New taxon";
		} else {
			return name.getTitleCache();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getPersistable()
	 */
	/**
	 * <p>getPersistable</p>
	 *
	 * @return a {@link org.eclipse.ui.IPersistableElement} object.
	 */
	@Override
    public IPersistableElement getPersistable() {
//		if(CdmStore.isActive()){
//			TaxonNode test = CdmStore.getTaxonTreeService().getTaxonNodeByUuid(taxonNode.getUuid());
//			boolean isPersistable = CdmStore.getTaxonTreeService().getTaxonNodeByUuid(taxonNode.getUuid()) != null;
//			if (isPersistable) {
//				return this;
//			}
//		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	/**
	 * <p>getToolTipText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getToolTipText() {
		return getName();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	/** {@inheritDoc} */
	@Override
    public Object getAdapter(Class adapter) {

		if (adapter == Taxon.class) {
			return taxonNode.getTaxon();
		}

		if (adapter == TaxonNode.class) {
			return taxonNode;
		}

		return null;
	}

	/**
	 * {@inheritDoc}
	 *
	 * Overrides equals to ensure that a taxon can only be edited by
	 * one editor at a time.
	 */
	@Override
    public boolean equals(Object obj) {
		if (TaxonEditorInput.class.equals(obj.getClass())
				&& getTaxon() != null
				&& getTaxon().equals(((TaxonEditorInput) obj).getTaxon())){
			if(((TaxonEditorInput) obj).getInitiallySelectedTaxonBase() != null){
				setInitiallySelectedTaxonBase(((TaxonEditorInput) obj).getInitiallySelectedTaxonBase());
			}
			return true;
		}
		return false;
	}

	/**
	 * <p>getTaxon</p>
	 *
	 * @return the taxon
	 */
	public Taxon getTaxon(){
		return taxonNode.getTaxon();
	}

	/**
	 * <p>Getter for the field <code>taxonNode</code>.</p>
	 *
	 * @return the taxonNode
	 */
	public TaxonNode getTaxonNode() {
		return taxonNode;
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	/**
	 * <p>getConversationHolder</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	@Override
    public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostCrudObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmCrudEvent)
	 */
	/** {@inheritDoc} */
	@Override
    public void update(CdmDataChangeMap events) {
		if(dataChangeBehavior == null){
			dataChangeBehavior = new TaxonEditorInputDataChangeBehaviour(this);
		}

		DataChangeBridge.handleDataChange(events, dataChangeBehavior);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPersistableElement#getFactoryId()
	 */
	/**
	 * <p>getFactoryId</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getFactoryId() {
		return TaxonEditorInputFactory.getFactoryId();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPersistable#saveState(org.eclipse.ui.IMemento)
	 */
	/** {@inheritDoc} */
	@Override
    public void saveState(IMemento memento) {
		TaxonEditorInputFactory.saveState(memento, this);
	}


    /**
     * <p>Setter for the field <code>initiallySelectedTaxonBase</code>.</p>
     *
     * @param taxonBase a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
     */
    public void setInitiallySelectedTaxonBase(TaxonBase taxonBase) {
		this.initiallySelectedTaxonBase = taxonBase;
	}

	/**
	 * <p>Getter for the field <code>initiallySelectedTaxonBase</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
	 */
	public TaxonBase getInitiallySelectedTaxonBase() {
		return initiallySelectedTaxonBase;
	}

	@Override
	public String toString() {
		return String.format("%s[%s]", this.getClass().getSimpleName(), getTaxon());
	}
}
