package eu.etaxonomy.taxeditor.editor.view.checklist.edit;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxViewerCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableColumn;

import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.view.checklist.ChecklistEditor;
import eu.etaxonomy.taxeditor.editor.view.checklist.ChecklistLabelProvider;
import eu.etaxonomy.taxeditor.store.CdmStore;

public final class DistributionEditingSupport extends EditingSupport {

    private ComboBoxViewerCellEditor cellEditor = null;
    private final TableViewer viewer;
    private final ChecklistEditor editor;
    private final IDescriptionService descriptionService;
    private final int columnIndex;

    private static final Logger logger = Logger.getLogger(DistributionEditingSupport.class);

    public DistributionEditingSupport(TableViewer viewer, ChecklistEditor checklistEditor, int columnIndex) {
        super(viewer);
        this.viewer = viewer;
        this.columnIndex = columnIndex;
        descriptionService = CdmStore.getService(IDescriptionService.class);
        editor = checklistEditor;
        cellEditor = new ComboBoxViewerCellEditor((Composite) viewer.getControl(), SWT.READ_ONLY);
        cellEditor.setLabelProvider(new LabelProvider());
        cellEditor.setContentProvider(new ArrayContentProvider());
        // TODO set input
        List<PresenceAbsenceTermBase> input = CdmStore.getTermManager()
                .getPreferredTerms(PresenceAbsenceTermBase.class);
        cellEditor.setInput(input);
    }

    @Override
    protected CellEditor getCellEditor(Object element) {
        return cellEditor;
    }

    @Override
    protected boolean canEdit(Object element) {
        return true;
    }

    @Override
    protected Object getValue(Object element) {
        if (element instanceof Taxon) {
            Taxon taxon = (Taxon) element;
            Distribution distributionForColumn = getDistributionForColumn(taxon);
            if (distributionForColumn != null) {
                distributionForColumn.getStatus().getTitleCache();
            }
            // return PresenceAbsenceTerms
        }
        return null;
    }

    @Override
    protected void setValue(Object element, Object value) {
        if (element instanceof Taxon && value instanceof PresenceAbsenceTermBase) {
            Taxon taxon = (Taxon) element;
            PresenceAbsenceTermBase occurenceStatus = (PresenceAbsenceTermBase) value;
            Distribution distribution = getDistributionForColumn(taxon);
            if (distribution != null) {
                // change status for already exsiting distribution
                distribution.setStatus(occurenceStatus);
            } else {
                createDistributionForColumn(taxon, occurenceStatus);
            }
            // viewer.update(distribution, null);
            viewer.update(element, null);
            viewer.refresh();
            editor.setDirty(true);
            editor.changed(element);
            viewer.refresh();
        }
    }

    private static final List<String> DESC_INIT_STRATEGY = Arrays.asList(new String[] { "descriptions",
            "descriptions.*", "description.state" });

    /**
     *
     * @param taxon
     * @return
     */
    private Distribution getDistributionForColumn(Taxon taxon) {
        List<TaxonDescription> listTaxonDescriptions = descriptionService.listTaxonDescriptions(taxon, null, null,
                null, null, null, DESC_INIT_STRATEGY);
        TableColumn column = viewer.getTable().getColumn(columnIndex);
        for (TaxonDescription td : listTaxonDescriptions) {
            for (DescriptionElementBase deb : td.getElements()) {
                if (deb instanceof Distribution) {
                    Distribution distribution = (Distribution) deb;
                    if (column.getText().equalsIgnoreCase(distribution.getArea().getTitleCache())) {
                        return distribution;
                    }
                }
            }
        }
        return null;
    }

    private void createDistributionForColumn(Taxon taxon, PresenceAbsenceTermBase occurenceStatus) {
        TableColumn column = viewer.getTable().getColumn(columnIndex);

        ChecklistLabelProvider labelProvider = new ChecklistLabelProvider();
        List<DefinedTermBase<?>> namedAreas = labelProvider.getNamedAreas();
        SortedSet<DefinedTermBase<?>> terms = (SortedSet<DefinedTermBase<?>>) labelProvider.getTermsOrderedByLabels(
                namedAreas, CdmStore.getDefaultLanguage());

        if (terms != null) {
            for (DefinedTermBase term : terms) {
                if (column.getText().equalsIgnoreCase(term.getTitleCache())) {
                    NamedArea area = (NamedArea) term;
                    Distribution distribution = Distribution.NewInstance(area, occurenceStatus);
                    Set<TaxonDescription> descriptions = taxon.getDescriptions();
                    if (descriptions != null) {
                        for (TaxonDescription desc : descriptions) {
                            // add to first taxon description
                            desc.addElement(distribution);
                            break;
                        }
                    } else {// there are no TaxonDescription yet.
                        TaxonDescription td = TaxonDescription.NewInstance(taxon);
                        td.addElement(distribution);
                    }
                }
            }
        }

        return;
    }
}